const Config = Object.freeze({
    // Maximum search shortcuts available from index
    MAX_SEARCH_SHORTCUTS: 12,
    REQUEST_TIMEOUT: 2500,
    RESULTS_PER_PAGE: 8,
    ALLOW_SEARCH_SUGGESTIONS: true,
});

export default Config;
