import { css } from 'emotion';

const actionsContainer = css`
    grid-area: actions;
`;

const actionsBtnWithNoSpacing = css`
    padding: 0;
`;

export { actionsContainer, actionsBtnWithNoSpacing };
