export const LOAD_RESULT_INIT = 'LOAD_RESULT_INIT';
export const LOAD_RESULT_PENDING = 'LOAD_RESULT_PENDING';
export const LOAD_RESULT_SUCCESS = 'LOAD_RESULT_SUCCESS';
export const LOAD_RESULT_ERROR = 'LOAD_RESULT_ERROR';
export const CLEAR_RESULTS = 'CLEAR_RESULTS';
